// g_local.h

#define GAME_INCLUDE

#include "../common/q_shared.h"
#include "../server/game.h"

gameExport_t *GetGameApi (gameImport_t *import);


class Entity: public SEntity
{
public:
	float		freetime;			// g_time when the object was freed
	double		lastruntime;		// g_time when SV_RunEntity was last
									// called for this edict
	char		*classname;
	int			spawnflags;
	float		ltime;
	int			movetype;
	Angles		avelocity;
	float		nextthink;
	void		(*think)(Entity *self);
	void		(*touch)(Entity *self, Entity *other);
	void		(*blocked)(Entity *self, Entity *other);
	void		(*use)(Entity *self, Entity *other, Entity *activator);
	void		(*pain)(Entity *self, Entity *other);
	void		(*die)(Entity *self, Entity *inflictor, Entity *attacker/*, int damage, Vec3 point*/);
	Entity		*groundentity;
//@@	int		health;
	int			weapon;
	int			ammo_shells;
	int			ammo_nails;
	int			ammo_rockets;
	int			ammo_cells;
	int			items;
	int			takedamage;
	int			chain;
	int			deadflag;
	int			max_health;
	float		armortype;
	int			armorvalue;
	int			waterlevel;
	int			watertype;
	float		ideal_yaw;
	char		*target;
	char		*targetname;
	char		*killtarget;
	Vec3		movedir;
	int			sounds;
	char		*noise;
	char		*noise1;
	char		*noise2;
	char		*noise3;
	char		*noise4;
	int			style;

	Entity		*enemy;
	Entity		*aiment;

	// timing variables
	float		wait;
	float		delay;			// before firing targets

	Vec3		oldorigin;		// only used by the secret door
	Vec3		pos1, pos2;
	float		speed;
	float		dmg;
	float		lip;
	int			state;
	Vec3		finaldest;
	void		(*think1)(Entity *self);
	float		attack_finished;
	Entity		*trigger_field;

	// func_plat
	float		height;

	// secret door
	float		t_length, t_width;
	Vec3		dest1, dest2;

	Vec3		mangle;
	char		*map;
	int			worldtype;

	// health boxes
	int			healtype;
	int			healamount;

	// ammo
	int			aflag;

	// players
	qboolean	player;
	int			walkframe;
//@@	int			frags;
	char		netname[16];
	qboolean	spectator;
	int			buttons;
	int			impulse;
	char		*weaponmodel;
//@@	int			weaponframe;
	int			currentammo;
	qboolean	axhitme;
	float		pain_finished;
	float		fly_sound;

	operator SEntity * () const { return (SEntity *)this; }
	operator SEntity& () const { return *(SEntity *)this; }
};

//============================================================================

extern gameImport_t gi;
extern gameExport_t	globals;
extern double g_time;
extern Entity g_edicts[MAX_EDICTS];
extern stats_t g_stats[MAX_CLIENTS];

extern CVar *g_deathmatch;
extern CVar *g_skill;
extern CVar *g_cheats;
extern CVar *g_sv_gravity, *g_sv_maxvelocity;
extern int current_skill;

//=================================================================

// sound channels
// channel 0 never willingly overrides
// other channels (1-7) always override a playing sound on that channel
#define CHAN_AUTO		0
#define CHAN_WEAPON 	1
#define CHAN_VOICE		2
#define CHAN_ITEM		3
#define CHAN_BODY		4
#define CHAN_NO_PHS_ADD	8	// ie: CHAN_BODY+CHAN_NO_PHS_ADD

#define ATTN_NONE		0
#define ATTN_NORM		1
#define ATTN_IDLE		2
#define ATTN_STATIC 	3

// edict->movetype
#define	MOVETYPE_NONE			0		// never moves
#define	MOVETYPE_WALK			1		// gravity
#define	MOVETYPE_STEP			2		// gravity, special edge handling
#define	MOVETYPE_FLY			3
#define	MOVETYPE_TOSS			4		// gravity
#define	MOVETYPE_BOUNCE			5
#define	MOVETYPE_PUSH			6		// no clip to world, push and crush
#define	MOVETYPE_NOCLIP			7

// edict->deadflag
#define	DEAD_NO					0
#define	DEAD_DYING				1
#define	DEAD_DEAD				2
#define DEAD_RESPAWNABLE		3

// edict->takedamage
#define	DAMAGE_NO				0
#define	DAMAGE_YES				1
#define	DAMAGE_AIM				2


//=================================================================

// memory tags to allow dynamic memory to be cleaned up
#define	TAG_GAME	765		// clear when unloading the dll
#define	TAG_LEVEL	766		// clear when loading a new level

// death types
typedef enum {
	DT_OTHER,
	DT_AXE,
	DT_SHOTGUN,
	DT_SUPER_SHOTGUN,
	DT_NAILGUN,
	DT_SUPER_NAILGUN,
	DT_GRENADE,
	DT_ROCKET,
	DT_LIGHTNING,
	DT_DISCHARGE,
	DT_TELEDEATH,
	DT_TELEDEATH2,
	DT_TELEDEATH3,
	DT_FALLING,
	DT_WATER,
	DT_SLIME,
	DT_LAVA,
	DT_SQUISH,
	DT_SELFWATER,
} deathtype_t;

//=================================================================

#define	FOFS(x) (int)&(((Entity *)0)->x)

enum fieldtype_t {
	F_INT, 
	F_FLOAT,
	F_STRING,
	F_VECTOR,
	F_ANGLES,		// like vector, but fix up Quake's "pitch yaw roll" order
	F_EDICT,
	F_ANGLEHACK,
	F_IGNORE
};

struct field_t {
	char	*name;
	int		ofs;
	fieldtype_t type;
	int		flags;
};

extern field_t fields[];

//=================================================================
// misc stuff

#define G_NUM_FOR_EDICT(e) (((byte *)(e) - (byte *)g_edicts)/sizeof(Entity))

#define world (g_edicts)

#define remove(ent) G_FreeEntity(ent)
#define objerror gi.error

// could be just a macro or inline
void setsize (Entity *ent, float mins_x, float mins_y, float mins_z,
			  float maxs_x, float maxs_y, float maxs_z);

inline float random() { return rand() / (RAND_MAX + 1.0); }
inline float crandom() { return 2.0 * random() - 1.0; }

//=================================================================

// functions exported by game

void G_Init (void);
void G_Shutdown (void);
void G_SpawnEntities (char *mapname, char *entstring);

bool G_ClientConnect (Entity *ent, char *userinfo);
void G_ClientUserinfoChanged (Entity *ent, char *userinfo);
void G_ClientBegin (Entity *ent);
void G_ClientDisconnect (Entity *ent);
bool G_ClientCommand (Entity *ent);
void G_ClientThink (Entity *ent, usercmd_t *cmd);

//=================================================================

void G_RunFrame (double time);
void G_ServerCommand (void);

// g_spawn.c
Entity *G_Spawn (void);
void G_FreeEntity (Entity *ent);

// g_main.c
void G_PrecacheModel (char *name);
void G_PrecacheSound (char *name);
void G_MakeStatic (Entity *ent);
void G_StaticSound (Vec3 &origin, char *name, float volume, float attenuation);
void G_LinkEntity (Entity *ent);
void G_UnlinkEntity (Entity *ent);
void G_Lightstyle (int style, char *string);
void G_Centerprint (Entity *ent, char *text);
void G_Sound (Entity *ent, int channel, char *name);	// full volume, normal attenuation
void G_SoundEx (Entity *ent, int channel, char *name, float volume, float attenuation);
int G_EntitiesInBox (Vec3 &mins, Vec3 &maxs, Entity **list, int maxcount);
int G_TriggersInBox (Vec3 &mins, Vec3 &maxs, Entity **list, int maxcount);

// g_utils.c
Entity *G_Find (Entity *from, int fieldofs, char *match);
void G_UseTargets (Entity *self, Entity *activator);
void G_SetMovedir (Angles &angles, Vec3 &movedir);
void G_TouchTriggers (Entity *ent);

// g_combat.c
void G_Damage (Entity *targ, Entity *inflictor, Entity *attacker, float damage, deathtype_t deathtype);
void G_RadiusDamage(Entity *inflictor, Entity *attacker, float damage, Entity *ignore, deathtype_t deathtype);

// g_weapon.cpp
void W_WeaponFrame (Entity *self, int buttons, int impulse);
void W_SetCurrentAmmo (Entity *self);
int W_BestWeapon (Entity *self);

// g_phys.cpp
void G_RunNewmis (Entity *ent, double time);

// g_player.cpp
void player_run (Entity *self);
