// g_main.c

#include "g_local.h"

gameImport_t	gi;
gameExport_t	globals;

double			g_time;
Entity g_edicts[MAX_EDICTS];

stats_t		g_stats[MAX_CLIENTS];

CVar *g_deathmatch, *g_skill, *g_cheats;
CVar *g_sv_gravity, *g_sv_maxvelocity;
int current_skill;


void G_LinkEntity (Entity *ent)
{
	gi.linkentity (ent);
}

void G_UnlinkEntity (Entity *ent)
{
	gi.unlinkentity (ent);
}

void G_MakeStatic (Entity *ent)
{
	gi.spawnstatic (ent);
	G_FreeEntity (ent);
}

void G_StaticSound (Vec3 &origin, char *name, float volume, float attenuation)
{
	gi.staticsound (origin, name, volume, attenuation);
}

void G_PrecacheModel (char *name)
{
	gi.precache_model (name);
}

void G_PrecacheSound (char *name)
{
	gi.precache_sound (name);
}

void G_Centerprint (Entity *ent, char *text)
{
	gi.centerprint (ent, text);
}

void G_Sound (Entity *ent, int channel, char *name)
{
	if (name)
		gi.sound (ent, channel, name, 1, ATTN_NORM);
}

void G_SoundEx (Entity *ent, int channel, char *name, float volume, float attenuation)
{
	if (name)
		gi.sound (ent, channel, name, volume, attenuation);
}

void G_Lightstyle (int style, char *string)
{
	gi.lightstyle (style, string);
}

int G_EntitiesInBox (Vec3 &mins, Vec3 &maxs, Entity **list, int maxcount)
{
	return gi.BoxEntities (mins, maxs, list, maxcount, AREA_SOLID);
}

int G_TriggersInBox (Vec3 &mins, Vec3 &maxs, Entity **list, int maxcount)
{
	return gi.BoxEntities (mins, maxs, list, maxcount, AREA_TRIGGERS);
}


void setsize (Entity *ent, float mins_x, float mins_y, float mins_z,
			  float maxs_x, float maxs_y, float maxs_z)
{
	ent->mins[0] = mins_x;
	ent->mins[1] = mins_y;
	ent->mins[2] = mins_z;
	ent->maxs[0] = maxs_x;
	ent->maxs[1] = maxs_y;
	ent->maxs[2] = maxs_z;
}

#ifndef GAME_HARD_LINKED

// these are only here so that functions in mathlib.cpp
// and q_shared.cpp can link

void Com_Printf (char *fmt, ...) {
	va_list		argptr;
	char		msg[1024];
	
	va_start (argptr, fmt);
	vsprintf (msg, fmt, argptr);
	va_end (argptr);

	gi.dprintf (msg);
}

void Com_DPrintf (char *fmt, ...) {
	va_list		argptr;
	char		msg[1024];

	// TODO: check developer cvar

	va_start (argptr, fmt);
	vsprintf (msg, fmt, argptr);
	va_end (argptr);

	gi.dprintf (msg);
}

void Sys_Error (char *fmt, ...) {
	va_list		argptr;
	char		msg[1024];

	va_start (argptr, fmt);
	vsprintf (msg, fmt, argptr);
	va_end (argptr);

	gi.error (msg);
}
#endif	// GAME_HARD_lINKED


/*
** This will be called when the dll is first loaded, which
** only happens when a new game is started or a save game
** is loaded.
*/
void G_Init () {
	gi.dprintf ("==== InitGame ====\n");

	g_deathmatch = gi.Cvar_Get ("deathmatch");
	g_skill = gi.Cvar_Get ("skill");
	g_cheats = gi.Cvar_Get ("sv_cheats");
	g_sv_gravity = gi.Cvar_Get ("sv_gravity");
	g_sv_maxvelocity = gi.Cvar_Get ("sv_maxvelocity");

	globals.edicts = g_edicts;
	globals.edict_size = sizeof(Entity);
	globals.max_edicts = MAX_EDICTS;
	globals.num_edicts = MAX_CLIENTS + 1;	// hmm?
}

void G_Shutdown () {
	gi.dprintf ("==== ShutdownGame ====\n");
}


gameExport_t *GetGameAPI (gameImport_t *import)
{
	gi = *import;

	globals.apiversion = GAME_API_VERSION;
	globals.Init = G_Init;
	globals.Shutdown = G_Shutdown;
	globals.SpawnEntities = G_SpawnEntities;
	globals.ClientConnect = G_ClientConnect;
	globals.ClientBegin = G_ClientBegin;
	globals.ClientUserinfoChanged = G_ClientUserinfoChanged;
	globals.ClientDisconnect = G_ClientDisconnect;
	globals.ClientCommand = G_ClientCommand;
	globals.ClientThink = G_ClientThink;

	globals.RunFrame = G_RunFrame;

	return &globals;
}
