// g_misc.c

#include "g_local.h"


/*QUAKED info_null (0 0.5 0) (-4 -4 -4) (4 4 4)
Used as a positional target for spotlights, etc.
*/
void SP_info_null (Entity *self)
{
	remove (self);
}


/*QUAKED info_notnull (0 0.5 0) (-4 -4 -4) (4 4 4)
Used as a positional target for lightning.
*/
void SP_info_notnull (Entity *self)
{
	// FIXME: Took this code from Q2... What's it for? -- Tonik
	self->absmin = self->origin;
	self->absmax = self->origin;
}

//============================================================================

#define START_OFF	1

void Use_Light (Entity *self, Entity *other, Entity *activator)
{
	if (self->spawnflags & START_OFF)
	{
		G_Lightstyle (self->style, "m");
		self->spawnflags &= ~START_OFF;
	}
	else
	{
		G_Lightstyle (self->style, "a");
		self->spawnflags |= START_OFF;
	}
}

/*QUAKED light (0 1 0) (-8 -8 -8) (8 8 8) START_OFF
Non-displayed light.
Default light value is 300
Default style is 0
If targeted, it will toggle between on or off.
*/
void SP_light (Entity *self)
{
	if (!self->targetname)
	{       // inert light
		G_FreeEntity (self);
		return;
	}
	
	if (self->style >= 32)
	{
		self->use = Use_Light;
		if (self->spawnflags & START_OFF)
			G_Lightstyle (self->style, "a");
		else
			G_Lightstyle (self->style, "m");
	}
}

/*QUAKED light_fluoro (0 1 0) (-8 -8 -8) (8 8 8) START_OFF
Non-displayed light.
Default light value is 300
Default style is 0
If targeted, it will toggle between on or off.
Makes steady fluorescent humming sound
*/
void SP_light_fluoro (Entity *self)
{
	if (self->style >= 32)
	{
		self->use = Use_Light;
		if (self->spawnflags & START_OFF)
			G_Lightstyle (self->style, "a");
		else
			G_Lightstyle (self->style, "m");
	}
	
	G_PrecacheSound ("ambience/fl_hum1.wav");
	G_StaticSound (self->origin, "ambience/fl_hum1.wav", 0.5, ATTN_STATIC);
}

/*QUAKED light_fluorospark (0 1 0) (-8 -8 -8) (8 8 8)
Non-displayed light.
Default light value is 300
Default style is 10
Makes sparking, broken fluorescent sound
*/
void SP_light_fluorospark (Entity *ent)
{
	if (!ent->style)
		ent->style = 10;

	G_PrecacheSound ("ambience/buzz1.wav");
	G_StaticSound (ent->origin, "ambience/buzz1.wav", 0.5, ATTN_STATIC);
}

/*QUAKED light_globe (0 1 0) (-8 -8 -8) (8 8 8)
Sphere globe light.
Default light value is 300
Default style is 0
*/
void SP_light_globe (Entity *ent)
{
	G_PrecacheModel ("progs/s_light.spr");
	gi.setmodel(ent, "progs/s_light.spr");
	G_MakeStatic (ent);
}

void FireAmbient (Entity *ent)
{
	G_PrecacheSound ("ambience/fire1.wav");
// attenuate fast
	G_StaticSound (ent->origin, "ambience/fire1.wav", 0.5, ATTN_STATIC);
}


/*QUAKED light_torch_small_walltorch (0 .5 0) (-10 -10 -20) (10 10 20)
Short wall torch
Default light value is 200
Default style is 0
*/
void SP_light_torch_small_walltorch (Entity *ent)
{
	G_PrecacheModel ("progs/flame.mdl");
	gi.setmodel(ent, "progs/flame.mdl");
	FireAmbient (ent);
	G_MakeStatic (ent);
}

/*QUAKED light_flame_large_yellow (0 1 0) (-10 -10 -12) (12 12 18)
Large yellow flame ball
*/
void SP_light_flame_large_yellow (Entity *ent)
{
	G_PrecacheModel ("progs/flame2.mdl");
	gi.setmodel(ent, "progs/flame2.mdl");
	ent->frame = 1;
	FireAmbient (ent);
	G_MakeStatic (ent);
}

/*QUAKED light_flame_small_yellow (0 1 0) (-8 -8 -8) (8 8 8) START_OFF
Small yellow flame ball
*/
void SP_light_flame_small_yellow (Entity *ent)
{
	G_PrecacheModel ("progs/flame2.mdl");
	gi.setmodel(ent, "progs/flame2.mdl");
	FireAmbient (ent);
	G_MakeStatic (ent);
}

/*QUAKED light_flame_small_white (0 1 0) (-10 -10 -40) (10 10 40) START_OFF
Small white flame ball
*/
void SP_light_flame_small_white (Entity *ent)
{
	G_PrecacheModel ("progs/flame2.mdl");
	gi.setmodel(ent, "progs/flame2.mdl");
	FireAmbient (ent);
	G_MakeStatic (ent);
}


/*
==============================================================================

MOVETARGET CODE

The angle of the movetarget effects standing and bowing direction, but has no effect on movement, which always heads to the next target.
Also used as target for trains

"targetname"	must be present.  The name of this movetarget.
"target"		the next spot to move to.  If not present, stop here for good.
"pausetime"		number of seconds to spend standing or bowing for path_stand or path_bow

==============================================================================
*/

/*
=============
Touch_Movetarget

Something has bumped into a movetarget.  If it is a monster
moving towards it, change the next destination and continue.
==============
*/
#if 0
void Touch_Movetarget (Entity *self, Entity *other)
{
	if (other->movetarget != self)
		return;
	
	if (other->enemy)
		return;         // fighting, not following a path

	other->goalentity = other->movetarget = G_Find (NULL, FOFS(targetname), self->target);
	other->ideal_yaw = vectoyaw(other->goalentity->origin - other->origin);
	if (!other->movetarget)
	{
		other->pausetime = g_time + 999999;
		other->th_stand ();
		return;
	}
}
#endif

/*QUAKED path_corner (0.5 0.3 0) (-8 -8 -8) (8 8 8)
Monsters will continue walking towards the next target corner.
*/
void SP_path_corner (Entity *self)
{
	if (!self->targetname)
		objerror ("path_corner: no targetname");
		
	self->solid = SOLID_TRIGGER;
//	self->touch = Touch_Movetarget;
	setsize (self, -8, -8, -8, 8, 8, 8);
	G_LinkEntity (self);
}


/*
==============================================================================

SIMPLE BMODELS

==============================================================================
*/

void func_wall_use (Entity *self, Entity *other, Entity *activator)
{       // change to alternate textures
	self->frame = 1 - self->frame;
}

/*QUAKED func_wall (0 .5 .8) ?
This is just a solid wall if not inhibitted
*/
void SP_func_wall (Entity *self)
{
	self->movetype = MOVETYPE_PUSH;  // so it doesn't get pushed by anything
	self->solid = SOLID_BSP;
	self->use = func_wall_use;
	gi.setmodel(self, self->model);
}


/*QUAKED func_illusionary (0 .5 .8) ?
A simple entity that looks solid but lets you walk through it.
*/
void SP_func_illusionary (Entity *self)
{
	self->movetype = MOVETYPE_NONE;
	self->solid = SOLID_NOT;
	gi.setmodel(self, self->model);
	G_MakeStatic (self);
}

/*QUAKED func_episodegate (0 .5 .8) ? E1 E2 E3 E4
This bmodel will appear if the episode has already been completed, so players can't reenter it.
*/
void SP_func_episodegate (Entity *self)
{
	remove (self);
/*	if (!(serverflags & self->spawnflags))
		return;                 // can still enter episode

	self->movetype = MOVETYPE_PUSH;  // so it doesn't get pushed by anything
	self->solid = SOLID_BSP;
	self->use = func_wall_use;
	gi.setmodel(self, self->model);*/
}

/*QUAKED func_bossgate (0 .5 .8) ?
This bmodel appears unless players have all of the episode sigils.
*/
void SP_func_bossgate (Entity *self)
{
//	if ((serverflags & 15) == 15)
//		return;         // all episodes completed

	self->movetype = MOVETYPE_PUSH;  // so it doesn't get pushed by anything
	self->solid = SOLID_BSP;
	self->use = func_wall_use;
	gi.setmodel(self, self->model);
}

//============================================================================

/*QUAKED ambient_suck_wind (0.3 0.1 0.6) (-10 -10 -8) (10 10 8)
*/
void SP_ambient_suck_wind (Entity *self)
{
	G_PrecacheSound ("ambience/suck1.wav");
	G_StaticSound (self->origin, "ambience/suck1.wav", 1, ATTN_STATIC);
}

/*QUAKED ambient_drone (0.3 0.1 0.6) (-10 -10 -8) (10 10 8)
*/
void SP_ambient_drone (Entity *self)
{
	G_PrecacheSound ("ambience/drone6.wav");
	G_StaticSound (self->origin, "ambience/drone6.wav", 0.5, ATTN_STATIC);
}

/*QUAKED ambient_flouro_buzz (0.3 0.1 0.6) (-10 -10 -8) (10 10 8)
*/
void SP_ambient_flouro_buzz (Entity *self)
{
	G_PrecacheSound ("ambience/buzz1.wav");
	G_StaticSound (self->origin, "ambience/buzz1.wav", 1, ATTN_STATIC);
}
/*QUAKED ambient_drip (0.3 0.1 0.6) (-10 -10 -8) (10 10 8)
*/
void SP_ambient_drip (Entity *self)
{
	G_PrecacheSound ("ambience/drip1.wav");
	G_StaticSound (self->origin, "ambience/drip1.wav", 0.5, ATTN_STATIC);
}
/*QUAKED ambient_comp_hum (0.3 0.1 0.6) (-10 -10 -8) (10 10 8)
*/
void SP_ambient_comp_hum (Entity *self)
{
	G_PrecacheSound ("ambience/comp1.wav");
	G_StaticSound (self->origin, "ambience/comp1.wav", 1, ATTN_STATIC);
}
/*QUAKED ambient_thunder (0.3 0.1 0.6) (-10 -10 -8) (10 10 8)
*/
void SP_ambient_thunder (Entity *self)
{
	G_PrecacheSound ("ambience/thunder1.wav");
	G_StaticSound (self->origin, "ambience/thunder1.wav", 0.5, ATTN_STATIC);
}
/*QUAKED ambient_light_buzz (0.3 0.1 0.6) (-10 -10 -8) (10 10 8)
*/
void SP_ambient_light_buzz (Entity *self)
{
	G_PrecacheSound ("ambience/fl_hum1.wav");
	G_StaticSound (self->origin, "ambience/fl_hum1.wav", 0.5, ATTN_STATIC);
}
/*QUAKED ambient_swamp1 (0.3 0.1 0.6) (-10 -10 -8) (10 10 8)
*/
void SP_ambient_swamp1 (Entity *self)
{
	G_PrecacheSound ("ambience/swamp1.wav");
	G_StaticSound (self->origin, "ambience/swamp1.wav", 0.5, ATTN_STATIC);
}
/*QUAKED ambient_swamp2 (0.3 0.1 0.6) (-10 -10 -8) (10 10 8)
*/
void SP_ambient_swamp2 (Entity *self)
{
	G_PrecacheSound ("ambience/swamp2.wav");
	G_StaticSound (self->origin, "ambience/swamp2.wav", 0.5, ATTN_STATIC);
}

//============================================================================
