// g_utils.c

#include "g_local.h"


/*
=============
G_Find

Searches all active entities for the next one that holds
the matching string at fieldofs (use the FOFS() macro) in the structure.

Searches beginning at the edict after from, or the beginning if NULL
NULL will be returned if the end of the list is reached.

=============
*/
Entity *G_Find (Entity *from, int fieldofs, char *match)
{
	char	*s;

// FIXME: Check field type?
// Host_Error ("G_Find: field type != F_STRING");

	if (!from)
		from = g_edicts;
	else
		from++;

	for ( ; from < &g_edicts[globals.num_edicts] ; from++)
	{
		if (!from->inuse)
			continue;
		s = *(char **) ((byte *)from + fieldofs);
		if (!s)
			continue;
		if (striequal(s, match))
			return from;
	}

	return NULL;
}


void Think_Delay (Entity *ent)
{
	G_UseTargets (ent, ent->enemy);
	remove (ent);
}



/*
==============================
G_UseTargets

If ent.delay is set, a DelayedUse entity will be created that will actually
do the G_UseTargets after that many seconds have passed.

Centerprints any ent.message to the activator.

Removes all entities with a targetname that match ent.killtarget,
and removes them, so some events can remove other triggers.

Search for (string)targetname in all entities that
match (string)self.target and call their .use function

==============================
*/
void G_UseTargets (Entity *ent, Entity *activator)
{
	Entity *t;

//
// check for a delay
//
	if (ent->delay)
	{
	// create a temp object to fire at a later time
		t = G_Spawn();
		t->classname = "DelayedUse";
		t->nextthink = g_time + ent->delay;
		t->think = Think_Delay;
		t->enemy = activator;
		t->message = ent->message;
		t->killtarget = ent->killtarget;
		t->target = ent->target;
		return;
	}
	
	
//
// print the message
//
	if (ent->message && ent->message[0] && activator->player)
	{
		G_Centerprint (activator, ent->message);
		if (!ent->noise)
			G_Sound (activator, CHAN_VOICE, "misc/talk.wav");
	}

//
// kill the killtagets
//
	if (ent->killtarget)
	{
		t = NULL;
		while ((t = G_Find (t, FOFS(targetname), ent->killtarget)))
		{
			remove (t);
			if (!ent->inuse)
			{
				// dprintf("entity was removed while using killtargets\n");
				return;
			}
		}
	}
	
//
// fire targets
//
	if (ent->target)
	{
		t = NULL;
		while ((t = G_Find (t, FOFS(targetname), ent->target)))
		{
			if (t == ent)
			{
				// dprintf ("WARNING: Entity used itself\n");
			}
			if (t->use)
			{
				t->use (t, ent, activator);
				if (!ent->inuse)
				{
					// dprintf("entity was removed while using targets\n");
					return;
				}
			}
		}
	}
}


/*
==============================
G_SetMovedir

QuakeEd only writes a single float for angles (bad idea), so up and down are
just constant angles.
==============================
*/
void G_SetMovedir (Angles &angles, Vec3 &movedir)
{
	if (angles == Angles(-1, 0, 0))
		movedir = Vec3(0, 0, 1);	// up
	else if (angles == Angles(-2, 0, 0)) {
		movedir = Vec3(0, 0, -1);	// down
	} else
		AngleVectors (angles, movedir);

	angles.Clear();
}


/*
============
G_TouchTriggers
============
*/
void G_TouchTriggers (Entity *ent)
{
	int		i, num;
	Entity	*touch[MAX_EDICTS], *hit;

	num = G_TriggersInBox (ent->absmin, ent->absmax, touch,	MAX_EDICTS);

	// be careful, it is possible to have an entity in this
	// list removed (killtriggered) before we get to it
	for (i=0 ; i<num ; i++)
	{
		hit = touch[i];
		if (!hit->inuse)
			continue;
		if (!hit->touch)
			continue;
		hit->touch (hit, ent);
	}
}
