// g_weapon.cpp

#include "g_local.h"

void player_axe1 (Entity *self);
void player_axeb1 (Entity *self);
void player_axec1 (Entity *self);
void player_axed1 (Entity *self);
void player_shot1 (Entity *self);
void player_nail1 (Entity *self);
void player_light1 (Entity *self);
void player_rocket1 (Entity *self);

//=============================================================================

void W_FireAxe (Entity *self)
{
	Vec3 forward;
	AngleVectors (self->viewangles, forward);

	Vec3 source = self->origin + Vec3(0, 0, 16);
	trace_t trace = gi.trace (source, vec3_origin, vec3_origin,
		source + forward*64, MOVE_NORMAL, self);
	if (trace.fraction == 1.0)
		return;
	
	Vec3 org = trace.endpos - forward*4;

	if (((Entity *)trace.ent)->takedamage)
	{
		((Entity *)trace.ent)->axhitme = true;
//		SpawnBlood (org, 20);
		if (g_deathmatch->value > 3)
			G_Damage ((Entity *)trace.ent, self, self, 75, DT_AXE);
		else
			G_Damage ((Entity *)trace.ent, self, self, 20, DT_AXE);
	}
	else
	{       // hit wall
		G_Sound (self, CHAN_WEAPON, "player/axhit2.wav");

/*		WriteByte (MSG_MULTICAST, SVC_TEMPENTITY);
		WriteByte (MSG_MULTICAST, TE_GUNSHOT);
		WriteByte (MSG_MULTICAST, 3);
		WriteCoord (MSG_MULTICAST, org_x);
		WriteCoord (MSG_MULTICAST, org_y);
		WriteCoord (MSG_MULTICAST, org_z);
		multicast (org, MULTICAST_PVS); */
	}
}

//=============================================================================

void GrenadeExplode (Entity *self)
{
	G_RadiusDamage (self, self->owner, 120, NULL, DT_GRENADE);

	gi.event_txyz (TE_EXPLOSION, self->origin, MULTICAST_PHS);

	remove (self);
}

void GrenadeTouch (Entity *self, Entity *other)
{
	if (other == self->owner)
		return;         // don't explode on owner

	if (other->takedamage == DAMAGE_AIM) {
		GrenadeExplode (self);
		return;
	
	}
	G_Sound (self, CHAN_WEAPON, "weapons/bounce.wav");
	if (self->velocity.IsNull())
		self->avelocity.Clear();
}


void W_FireGrenade (Entity *self)
{       
	if (g_deathmatch->value != 4)
		self->currentammo = self->ammo_rockets = self->ammo_rockets - 1;
	
	G_Sound (self, CHAN_WEAPON, "weapons/grenade.wav");

//	msg_entity = self;
//	WriteByte (MSG_ONE, SVC_SMALLKICK);

	Entity *newmis = G_Spawn();
	newmis->classname = "grenade";
	newmis->owner = self;
	newmis->movetype = MOVETYPE_BOUNCE;
	newmis->solid = SOLID_BBOX;
	newmis->effects = EF_GRENADE;
		
	// set newmis speed     
	Vec3 angles = self->viewangles;
	if (angles[PITCH] < -70)
		angles[PITCH] = -70;
	Vec3 forward, right, up;
	AngleVectors (angles, forward, right, up);

	newmis->velocity = forward*600 + up * 200 + right*(10*crandom()) + up*(10*crandom());

	newmis->avelocity = Vec3(300, -300, 300);
	vectoangles (newmis->velocity, newmis->angles);
	
	newmis->touch = GrenadeTouch;
	
// set newmis duration
	if (g_deathmatch->value == 4)
	{
		newmis->nextthink = g_time + 2.5;		
		self->attack_finished = g_time + 1.1;
//		self->health = self->health - 1;
		G_Damage (self, self, self->owner, 10, DT_GRENADE);
	}
	else
		newmis->nextthink = g_time + 2.5;

	newmis->think = GrenadeExplode;

	gi.setmodel (newmis, "progs/grenade.mdl");
	newmis->origin = self->origin;
	gi.linkentity (self);

	G_RunNewmis (newmis, 0.05);
}


//=============================================================================

void Touch_Missile (Entity *self, Entity *other)
{
	if (other == self->owner)
		return;         // don't explode on owner

	if (gi.pointcontents(self->origin) == CONTENTS_SKY) {
		remove(self);
		return;
	}

	if (other->health) {
		int dmg = 100 + rand() % 20;
		G_Damage (other, self, self->owner, dmg, DT_ROCKET);
	}

	// don't do radius damage to the other, because all the damage
	// was done in the impact
	G_RadiusDamage (self, self->owner, 120, other, DT_ROCKET);

	Normalize (&self->velocity);
	VectorMA (self->origin, -8, self->velocity, self->origin);

	gi.event_txyz (TE_EXPLOSION, self->origin, MULTICAST_PHS);

	remove(self);
}

void W_FireRocket (Entity *self)
{
	Entity	*newmis;
	Vec3	forward;

	if (g_deathmatch->value != 4)
		self->currentammo = self->ammo_rockets = self->ammo_rockets - 1;
	
	G_Sound (self, CHAN_WEAPON, "weapons/sgun1.wav");

//	msg_entity = self;
//	WriteByte (MSG_ONE, SVC_SMALLKICK);

	newmis = G_Spawn ();
	newmis->classname = "rocket";
	newmis->owner = self;
	newmis->movetype = MOVETYPE_FLY;
	newmis->solid = SOLID_BBOX;
	newmis->effects = EF_ROCKET;
		
// set newmis speed     
	AngleVectors (self->viewangles, forward);
	newmis->velocity = forward * 1000;
	newmis->angles = self->viewangles;
	
	newmis->touch = Touch_Missile;
	
// set newmis duration
	newmis->nextthink = g_time + 5;
	newmis->think = G_FreeEntity;

	gi.setmodel (newmis, "progs/missile.mdl");
	newmis->origin = self->origin + forward * 8.0;
	newmis->origin[2] += 16;
	gi.linkentity (self);

	G_RunNewmis (newmis, 0.05);
}

/*
===============================================================================
LIGHTNING
===============================================================================
*/

void LightningHit (Entity *ent, Entity *from)
{
/*	WriteByte (MSG_MULTICAST, SVC_TEMPENTITY);
	WriteByte (MSG_MULTICAST, TE_LIGHTNINGBLOOD);
	WriteCoord (MSG_MULTICAST, trace_endpos_x);
	WriteCoord (MSG_MULTICAST, trace_endpos_y);
	WriteCoord (MSG_MULTICAST, trace_endpos_z);
	multicast (trace_endpos, MULTICAST_PVS);
*/

	G_Damage (ent, from, from, 30, DT_LIGHTNING);
}

/*
=================
LightningDamage
=================
*/
void LightningDamage (Entity *self, Vec3 p1, Vec3 p2, Entity *from)
{
	Entity	*e1, *e2;
	Vec3	f;
	
	f = p2 - p1;
	VectorNormalize (f);
	f.x = 0 - f.y;
	f.y = f.x;
	f.z = 0;
	f = f * 16;

	e1 = e2 = world;

	trace_t trace = gi.trace (p1, vec3_origin, vec3_origin, p2, MOVE_NORMAL, self);

	Entity *trace_ent = (Entity *)trace.ent;
	if (trace_ent && trace_ent->takedamage) {
		LightningHit (trace_ent, from);

		// dm6 platform bug-o-feature
		if (self->player && self->groundentity == trace_ent)
				trace_ent->velocity.z += 400;
	}
	e1 = trace_ent;

	trace = gi.trace (p1 + f, vec3_origin, vec3_origin, p2 + f, MOVE_NORMAL, self);
	trace_ent = (Entity *)trace.ent;
	if (trace_ent && trace_ent != e1 && trace_ent->takedamage) {
		LightningHit (trace_ent, from);
	}
	e2 = trace_ent;

	trace = gi.trace (p1 - f, vec3_origin, vec3_origin, p2 - f, MOVE_NORMAL, self);
	trace_ent = (Entity *)trace.ent;
	if (trace_ent && trace_ent != e1 && trace_ent != e2 && trace_ent->takedamage) {
		LightningHit (trace_ent, from);
	}
}

void W_FireLightning (Entity *self)
{
	Vec3	org;
	int		cells;

	if (self->ammo_cells < 1)
	{
		self->weapon = W_BestWeapon (self);
		W_SetCurrentAmmo (self);
		return;
	}

// explode if under water
	if (self->waterlevel > 1)
	{
		if (g_deathmatch->value > 3)
		{
			if (random() <= 0.5) {
				G_Damage (self, self, self->owner, 4000, DT_SELFWATER);
			}
			else
			{
				cells = self->ammo_cells;
				self->ammo_cells = 0;
				W_SetCurrentAmmo (self);
				G_RadiusDamage (self, self, 35*cells, world, DT_OTHER);
				return;
			}
		}
		else
		{
			cells = self->ammo_cells;
			self->ammo_cells = 0;
			W_SetCurrentAmmo (self);
			G_RadiusDamage (self, self, 35*cells, world, DT_OTHER);
			return;
		}
	}

	// FIXME: is it such a good idea to use t_width here?
	if (self->t_width < g_time)
	{
		G_Sound (self, CHAN_WEAPON, "weapons/lhit.wav");
		self->t_width = g_time + 0.6;
	}

//	msg_entity = self;
//	WriteByte (MSG_ONE, SVC_SMALLKICK);

	if (g_deathmatch->value != 4)
		self->currentammo = self->ammo_cells = self->ammo_cells - 1;

	org = self->origin + Vec3(0, 0, 16);

	Vec3 forward;
	AngleVectors (self->viewangles, forward);
	trace_t trace = gi.trace (org, vec3_origin, vec3_origin, org + forward*600, MOVE_NORMAL, self);

	gi.te_lightning (self, org, trace.endpos);

	LightningDamage (self, self->origin, trace.endpos + forward*4, self);
}

/*
===============================================================================

PLAYER WEAPON USE

===============================================================================
*/

void W_SetCurrentAmmo (Entity *self)
{
	player_run(self);		// get out of any weapon firing states

	self->items &= ~(IT_SHELLS|IT_NAILS|IT_ROCKETS|IT_CELLS);
	
	if (self->weapon == IT_AXE)
	{
		self->currentammo = 0;
		self->weaponmodel = "progs/v_axe.mdl";
		self->weaponframe = 0;
	}
	else if (self->weapon == IT_SHOTGUN)
	{
		self->currentammo = self->ammo_shells;
		self->weaponmodel = "progs/v_shot.mdl";
		self->weaponframe = 0;
		self->items = self->items | IT_SHELLS;
	}
	else if (self->weapon == IT_SUPER_SHOTGUN)
	{
		self->currentammo = self->ammo_shells;
		self->weaponmodel = "progs/v_shot2.mdl";
		self->weaponframe = 0;
		self->items = self->items | IT_SHELLS;
	}
	else if (self->weapon == IT_NAILGUN)
	{
		self->currentammo = self->ammo_nails;
		self->weaponmodel = "progs/v_nail.mdl";
		self->weaponframe = 0;
		self->items = self->items | IT_NAILS;
	}
	else if (self->weapon == IT_SUPER_NAILGUN)
	{
		self->currentammo = self->ammo_nails;
		self->weaponmodel = "progs/v_nail2.mdl";
		self->weaponframe = 0;
		self->items = self->items | IT_NAILS;
	}
	else if (self->weapon == IT_GRENADE_LAUNCHER)
	{
		self->currentammo = self->ammo_rockets;
		self->weaponmodel = "progs/v_rock.mdl";
		self->weaponframe = 0;
		self->items = self->items | IT_ROCKETS;
	}
	else if (self->weapon == IT_ROCKET_LAUNCHER)
	{
		self->currentammo = self->ammo_rockets;
		self->weaponmodel = "progs/v_rock2.mdl";
		self->weaponframe = 0;
		self->items = self->items | IT_ROCKETS;
	}
	else if (self->weapon == IT_LIGHTNING)
	{
		self->currentammo = self->ammo_cells;
		self->weaponmodel = "progs/v_light.mdl";
		self->weaponframe = 0;
		self->items = self->items | IT_CELLS;
	}
	else
	{
		self->currentammo = 0;
		self->weaponmodel = "";
		self->weaponframe = 0;
	}
}

// never returns grenade or rocket launcher
int W_BestWeapon (Entity *self)
{
	int it = self->items;

	if (self->waterlevel <= 1 && self->ammo_cells >= 1 && (it & IT_LIGHTNING) )
		return IT_LIGHTNING;
	else if(self->ammo_nails >= 2 && (it & IT_SUPER_NAILGUN) )
		return IT_SUPER_NAILGUN;
	else if(self->ammo_shells >= 2 && (it & IT_SUPER_SHOTGUN) )
		return IT_SUPER_SHOTGUN;
	else if(self->ammo_nails >= 1 && (it & IT_NAILGUN) )
		return IT_NAILGUN;
	else if(self->ammo_shells >= 1 && (it & IT_SHOTGUN)  )
		return IT_SHOTGUN;
		
	return IT_AXE;
}

bool W_CheckNoAmmo (Entity *self)
{
	if (self->currentammo > 0)
		return true;

	if (self->weapon == IT_AXE)
		return true;
	
	self->weapon = W_BestWeapon (self);

	W_SetCurrentAmmo (self);
	
// drop the weapon down
	return false;
}


void W_Attack (Entity *self)
{
	if (self->weapon == IT_AXE)
	{
		self->attack_finished = g_time + 0.5;
		G_Sound (self, CHAN_WEAPON, "weapons/ax1.wav");
		switch (rand()%4) {
			case 0:	player_axe1 (self); break;
			case 1: player_axeb1 (self); break;
			case 2: player_axec1 (self); break;
			default: player_axed1 (self); break;
		}
	}
	else if (self->weapon == IT_GRENADE_LAUNCHER)
	{
		player_rocket1 (self);
		self->attack_finished = g_time + 0.6;
		W_FireGrenade (self);
	}
	else if (self->weapon == IT_ROCKET_LAUNCHER)
	{
		if (self->ammo_rockets <= 0)
			return;
		player_rocket1 (self);
		self->attack_finished = g_time + 0.8;
		W_FireRocket (self);
	}
	else if (self->weapon == IT_LIGHTNING)
	{
		self->attack_finished = g_time + 0.1;
		G_Sound (self, CHAN_AUTO, "weapons/lstart.wav");
		player_light1 (self);
	}
}

void W_ChangeWeapon (Entity *self, int impulse)
{
	int fl;
	int it = self->items;
	bool noammo = false;

	switch (impulse) {
	case 1:
		fl = IT_AXE;
		break;
	case 2:
		fl = IT_SHOTGUN;
		if (self->ammo_shells < 1)
			noammo = true;
		break;
	case 3:
		fl = IT_SUPER_SHOTGUN;
		if (self->ammo_shells < 2)
			noammo = true;
		break;
	case 4:
		fl = IT_NAILGUN;
		if (self->ammo_nails < 1)
			noammo = true;
		break;
	case 5:
		fl = IT_SUPER_NAILGUN;
		if (self->ammo_nails < 2)
			noammo = true;
		break;
	case 6:
		fl = IT_GRENADE_LAUNCHER;
		if (self->ammo_rockets < 1)
			noammo = true;
		break;
	case 7:
		fl = IT_ROCKET_LAUNCHER;
		if (self->ammo_rockets < 1)
			noammo = true;
		break;
	case 8:
		fl = IT_LIGHTNING;
		if (self->ammo_cells < 1)
			noammo = true;
	}

	if (!(self->items & fl)) {
		gi.cprintf (self, "no weapon.\n");
		return;
	}
	
	if (noammo) {
		gi.cprintf (self, "not enough ammo.\n");
		return;
	}

	self->weapon = fl;
	W_SetCurrentAmmo (self);
}

void W_WeaponFrame (Entity *self, int buttons, int impulse)
{
	if (impulse)
		self->impulse = impulse;

	if (self->health <= 0 || g_time < self->attack_finished)
		return;

	impulse = self->impulse;

	if (impulse >= 1 && impulse <= 8)
		W_ChangeWeapon (self, impulse);

	self->impulse = 0;

	if (self->buttons & BUTTON_ATTACK)
		W_Attack (self);
}

